<?php

/**
 * Expose active contexts as a context condition.
 */
class context_condition_context extends context_condition_path {
  function execute() {
    if (!$this->condition_used()) {
     return;
    }

    // Contexts that were triggered by other conditions.
    $activeOld = context_active_contexts();

    // Get a list of contexts that may be activated by this condition.
    $contexts = array();
    // Prepare context values too.
    $activateable = array();
    foreach ($this->get_contexts() as $context) {
      if (!isset($activeOld[$context->name]) && ($values = $this->fetch_from_context($context, 'values'))) {
        $contexts[$context->name] = $context;
        $activateable[$context->name] = $values;
      }
    }
    $activeOld = array_keys($activeOld);
    $activeNew = $this->calculateActiveContexts($activeOld, $activateable);

    // Trigger all necessary contexts.
    foreach ($activeNew as $name) {
      $this->condition_met($contexts[$name]);
    }
  }

  /**
   * Calculate the list of contexts that we need to trigger.
   *
   * @param array $activeOld
   *   machine names of contexts that are already active
   * @param array $activateable
   *   associative array of condition values keyed by context name.
   *   listing all contexts that use this condition.
   * @return array
   *   machine names of all contexts that meet their conditions.
   */
  protected function calculateActiveContexts($activeOld, $activateable) {
    // Keep a list of already visited states.
    $knownStates = array();
    $activeNew = array();
    $knownKey = md5(implode('|', $activeNew));

    // Keep going until we hit an already visited state. This means either
    // - We found a stable state.
    // - We found a circle.
    while (!isset($knownStates[$knownKey])) {
      $knownStates[$knownKey] = 1;

      $activeStep = array_merge($activeOld, $activeNew);
      $activeNew = array();
      foreach ($activateable as $name => $values) {
        if ($this->match($activeStep, $values)) {
          $activeNew[] = $name;
        }
      }
      $knownKey = md5(implode('|', $activeNew));
    }

    return $activeNew;
  }
}
